@@ -153,6 +153,30 @@ QString intToRoman(int number)
     return result;
 }
 
+bool copyFile(QFile& source, QFile& destination)
+{
+    const qint64 maxSize = 4096;
+    qint64 size = -1;
+
+    QScopedArrayPointer< char > buffer(new char[maxSize]);
+
+    do
+    {
+        if((size = source.read(buffer.data(), maxSize)) < 0)
+        {
+            return false;
+        }
+
+        if(destination.write(buffer.data(), size) < 0)
+        {
+            return false;
+        }
+    }
+    while(size > 0);
+
+    return true;
+}
+
 #ifdef WITH_CUPS
 
 struct RemovePpdFileDeleter
@@ -190,7 +214,7 @@ int addCMYKorRGBColorModel(cups_dest_t* dest, int num_options, cups_option_t** o
 
     for(int index = 0; index < colorModel->num_choices; ++index)
     {
-        if(qstrncmp(colorModel->choices[index].choice, "CMYK", 4) == 0)
+        if(qstrcmp(colorModel->choices[index].choice, "CMYK") == 0)
         {
             return cupsAddOption("ColorModel", "CMYK", num_options, options);
         }
@@ -198,7 +222,7 @@ int addCMYKorRGBColorModel(cups_dest_t* dest, int num_options, cups_option_t** o
 
     for(int index = 0; index < colorModel->num_choices; ++index)
     {
-        if(qstrncmp(colorModel->choices[index].choice, "RGB", 3) == 0)
+        if(qstrcmp(colorModel->choices[index].choice, "RGB") == 0)
         {
             return cupsAddOption("ColorModel", "RGB", num_options, options);
         }
@@ -224,6 +248,22 @@ inline QRectF rectOfResult(const QModelIndex& index)
     return index.data(SearchModel::RectRole).toRectF();
 }
 
+inline void adjustScaleFactor(RenderParam& renderParam, qreal scaleFactor)
+{
+    if(!qFuzzyCompare(renderParam.scaleFactor(), scaleFactor))
+    {
+        renderParam.setScaleFactor(scaleFactor);
+    }
+}
+
+inline void setValueIfNotVisible(QScrollBar* scrollBar, int value)
+{
+    if(value < scrollBar->value() || value > scrollBar->value() + scrollBar->pageStep())
+    {
+        scrollBar->setValue(value);
+    }
+}
+
 void saveExpandedPaths(const QAbstractItemModel* model, QSet< QString >& paths, const QModelIndex& index, QString path)
 {
     path += index.data(Qt::DisplayRole).toString();
@@ -280,8 +320,7 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
     m_scaleMode(ScaleFactorMode),
     m_scaleFactor(1.0),
     m_rotation(RotateBy0),
-    m_invertColors(false),
-    m_convertToGrayscale(false),
+    m_renderFlags(0),
     m_highlightAll(false),
     m_rubberBandMode(ModifiersMode),
     m_pageItems(),
@@ -314,7 +353,7 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
     setAcceptDrops(false);
     setDragMode(QGraphicsView::ScrollHandDrag);
 
-    connectVerticalScrollBar();
+    reconnectVerticalScrollBar();
 
     m_thumbnailsScene = new QGraphicsScene(this);
 
@@ -375,8 +414,34 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
     m_scaleFactor = s_settings->documentView().scaleFactor();
     m_rotation = s_settings->documentView().rotation();
 
-    m_invertColors = s_settings->documentView().invertColors();
-    m_convertToGrayscale = s_settings->documentView().convertToGrayscale();
+    if(s_settings->documentView().invertColors())
+    {
+        m_renderFlags |= InvertColors;
+    }
+
+    if(s_settings->documentView().convertToGrayscale())
+    {
+        m_renderFlags |= ConvertToGrayscale;
+    }
+
+    if(s_settings->documentView().trimMargins())
+    {
+        m_renderFlags |= TrimMargins;
+    }
+
+    switch(s_settings->documentView().compositionMode())
+    {
+    default:
+    case DefaultCompositionMode:
+        break;
+    case DarkenWithPaperColorMode:
+        m_renderFlags |= DarkenWithPaperColor;
+        break;
+    case LightenWithPaperColorMode:
+        m_renderFlags |= LightenWithPaperColor;
+        break;
+    }
+
     m_highlightAll = s_settings->documentView().highlightAll();
 }
 
@@ -524,35 +589,7 @@ QString DocumentView::title() const
 
 QStringList DocumentView::openFilter()
 {
-    QStringList openFilter;
-    QStringList supportedFormats;
-
-#if defined(WITH_PDF) || defined(WITH_FITZ)
-
-    openFilter.append("Portable document format (*.pdf)");
-    supportedFormats.append("*.pdf");
-
-#endif // WITH_PDF // WITH_FITZ
-
-#ifdef WITH_PS
-
-    openFilter.append("PostScript (*.ps)");
-    openFilter.append("Encapsulated PostScript (*.eps)");
-    supportedFormats.append("*.ps *.eps");
-
-#endif // WITH_PS
-
-#ifdef WITH_DJVU
-
-    openFilter.append("DjVu (*.djvu *.djv)");
-    supportedFormats.append("*.djvu");
-    supportedFormats.append("*.djv");
-
-#endif // WITH_DJVU
-
-    openFilter.prepend(tr("Supported formats (%1)").arg(supportedFormats.join(" ")));
-
-    return openFilter;
+    return PluginHandler::openFilter();
 }
 
 QStringList DocumentView::saveFilter() const
@@ -680,53 +717,100 @@ void DocumentView::setRotation(Rotation rotation)
 
         emit rotationChanged(m_rotation);
 
-        s_settings->documentView().setRotation(rotation);
+        s_settings->documentView().setRotation(m_rotation);
     }
 }
 
-void DocumentView::setInvertColors(bool invertColors)
+void DocumentView::setRenderFlags(qpdfview::RenderFlags renderFlags)
 {
-    if(m_invertColors != invertColors)
+    if(m_renderFlags != renderFlags)
     {
-        m_invertColors = invertColors;
+        const qpdfview::RenderFlags changedFlags = m_renderFlags ^ renderFlags;
 
-        foreach(PageItem* page, m_pageItems)
+        m_renderFlags = renderFlags;
+
+        qreal left = 0.0, top = 0.0;
+        saveLeftAndTop(left, top);
+
+        prepareScene();
+        prepareView(left, top);
+
+        if(changedFlags.testFlag(InvertColors))
         {
-            page->setInvertColors(m_invertColors);
+            prepareBackground();
+
+            emit invertColorsChanged(invertColors());
+
+            s_settings->documentView().setInvertColors(invertColors());
         }
 
-        foreach(ThumbnailItem* page, m_thumbnailItems)
+        if(changedFlags.testFlag(ConvertToGrayscale))
         {
-            page->setInvertColors(m_invertColors);
+            emit convertToGrayscaleChanged(convertToGrayscale());
+
+            s_settings->documentView().setConvertToGrayscale(convertToGrayscale());
         }
 
-        prepareBackground();
+        if(changedFlags.testFlag(TrimMargins))
+        {
+            emit trimMarginsChanged(trimMargins());
 
-        emit invertColorsChanged(m_invertColors);
+            s_settings->documentView().setTrimMargins(trimMargins());
+        }
 
-        s_settings->documentView().setInvertColors(m_invertColors);
+        if(changedFlags.testFlag(DarkenWithPaperColor) || changedFlags.testFlag(LightenWithPaperColor))
+        {
+            emit compositionModeChanged(compositionMode());
+
+            s_settings->documentView().setCompositionMode(compositionMode());
+        }
+
+        emit renderFlagsChanged(m_renderFlags);
     }
 }
 
-void DocumentView::setConvertToGrayscale(bool convertToGrayscale)
+void DocumentView::setRenderFlag(qpdfview::RenderFlag renderFlag, bool enabled)
 {
-    if(m_convertToGrayscale != convertToGrayscale)
+    if(enabled)
     {
-        m_convertToGrayscale = convertToGrayscale;
+        setRenderFlags(m_renderFlags | renderFlag);
+    }
+    else
+    {
+        setRenderFlags(m_renderFlags & ~renderFlag);
+    }
+}
 
-        foreach(PageItem* page, m_pageItems)
-        {
-            page->setConvertToGrayscale(m_convertToGrayscale);
-        }
+CompositionMode DocumentView::compositionMode() const
+{
+    if(m_renderFlags.testFlag(DarkenWithPaperColor))
+    {
+        return DarkenWithPaperColorMode;
+    }
+    else if(m_renderFlags.testFlag(LightenWithPaperColor))
+    {
+        return LightenWithPaperColorMode;
+    }
+    else
+    {
+        return DefaultCompositionMode;
+    }
+}
 
-        foreach(ThumbnailItem* page, m_thumbnailItems)
-        {
-            page->setConvertToGrayscale(m_convertToGrayscale);
-        }
-
-        emit convertToGrayscaleChanged(m_convertToGrayscale);
-
-        s_settings->documentView().setConvertToGrayscale(m_convertToGrayscale);
+void DocumentView::setCompositionMode(CompositionMode compositionMode)
+{
+    switch(compositionMode)
+    {
+    default:
+    case DefaultCompositionMode:
+        setRenderFlags((renderFlags() & ~DarkenWithPaperColor) & ~LightenWithPaperColor);
+        break;
+    case DarkenWithPaperColorMode:
+        setRenderFlags((renderFlags() | DarkenWithPaperColor) & ~LightenWithPaperColor);
+        break;
+    case LightenWithPaperColorMode:
+        setRenderFlags((renderFlags() & ~DarkenWithPaperColor) | LightenWithPaperColor);
+        break;
     }
 }
 
@@ -946,45 +1030,41 @@ bool DocumentView::save(const QString& filePath, bool withChanges)
     QTemporaryFile temporaryFile;
     QFile file(filePath);
 
-    if(temporaryFile.open())
+    // Save document to temporary file...
+    if(!temporaryFile.open())
     {
-        temporaryFile.close();
-
-        if(m_document->save(temporaryFile.fileName(), withChanges))
-        {
-            if(temporaryFile.open())
-            {
-                if(file.open(QIODevice::WriteOnly | QIODevice::Truncate))
-                {
-                    const qint64 maxSize = 4096;
-                    qint64 size = -1;
-
-                    char* data = new char[maxSize];
-
-                    while(!temporaryFile.atEnd())
-                    {
-                        size = temporaryFile.read(data, maxSize);
-
-                        if(size == -1 || file.write(data, size) == -1)
-                        {
-                            delete[] data;
-                            return false;
-                        }
-                    }
-
-                    if(withChanges)
-                    {
-                        m_wasModified = false;
-                    }
-
-                    delete[] data;
-                    return true;
-                }
-            }
-        }
+        return false;
     }
 
-    return false;
+    temporaryFile.close();
+
+    if(!m_document->save(temporaryFile.fileName(), withChanges))
+    {
+        return false;
+    }
+
+    // Copy from temporary file to actual file...
+    if(!temporaryFile.open())
+    {
+        return false;
+    }
+
+    if(!file.open(QIODevice::WriteOnly | QIODevice::Truncate))
+    {
+        return false;
+    }
+
+    if(!copyFile(temporaryFile, file))
+    {
+        return false;
+    }
+
+    if(withChanges)
+    {
+        m_wasModified = false;
+    }
+
+    return true;
 }
 
 bool DocumentView::print(QPrinter* printer, const PrintOptions& printOptions)
@@ -1024,14 +1104,24 @@ void DocumentView::lastPage()
     jumpToPage(m_pages.count());
 }
 
-void DocumentView::jumpToPage(int page, bool trackChange, qreal changeLeft, qreal changeTop)
+void DocumentView::jumpToPage(int page, bool trackChange, qreal newLeft, qreal newTop)
 {
     if(page >= 1 && page <= m_pages.count())
     {
         qreal left = 0.0, top = 0.0;
         saveLeftAndTop(left, top);
 
-        if(m_currentPage != m_layout->currentPage(page) || qAbs(left - changeLeft) > 0.01 || qAbs(top - changeTop) > 0.01)
+        if(qIsNaN(newLeft))
+        {
+            newLeft = qBound(qreal(0.0), left, qreal(1.0));
+        }
+
+        if(qIsNaN(newTop))
+        {
+            newTop = qBound(qreal(0.0), top, qreal(1.0));
+        }
+
+        if(m_currentPage != m_layout->currentPage(page) || qAbs(left - newLeft) > 0.01 || qAbs(top - newTop) > 0.01)
         {
             if(trackChange)
             {
@@ -1043,7 +1133,7 @@ void DocumentView::jumpToPage(int page, bool trackChange, qreal changeLeft, qrea
 
             m_currentPage = m_layout->currentPage(page);
 
-            prepareView(changeLeft, changeTop, page);
+            prepareView(newLeft, newTop, false, page);
 
             emit currentPageChanged(m_currentPage, trackChange);
         }
@@ -1175,7 +1265,9 @@ void DocumentView::zoomIn()
 {
     if(scaleMode() != ScaleFactorMode)
     {
-        setScaleFactor(qMin(m_pageItems.at(m_currentPage - 1)->scaleFactor() * s_settings->documentView().zoomFactor(),
+        const qreal currentScaleFactor = m_pageItems.at(m_currentPage - 1)->renderParam().scaleFactor();
+
+        setScaleFactor(qMin(currentScaleFactor * s_settings->documentView().zoomFactor(),
                             s_settings->documentView().maximumScaleFactor()));
 
         setScaleMode(ScaleFactorMode);
@@ -1191,7 +1283,9 @@ void DocumentView::zoomOut()
 {
     if(scaleMode() != ScaleFactorMode)
     {
-        setScaleFactor(qMax(m_pageItems.at(m_currentPage - 1)->scaleFactor() / s_settings->documentView().zoomFactor(),
+        const qreal currentScaleFactor = m_pageItems.at(m_currentPage - 1)->renderParam().scaleFactor();
+
+        setScaleFactor(qMax(currentScaleFactor / s_settings->documentView().zoomFactor(),
                             s_settings->documentView().minimumScaleFactor()));
 
         setScaleMode(ScaleFactorMode);
@@ -1264,7 +1358,7 @@ void DocumentView::startPresentation()
     connect(this, SIGNAL(documentChanged()), presentationView, SLOT(close()));
 
     presentationView->setRotation(rotation());
-    presentationView->setInvertColors(invertColors());
+    presentationView->setRenderFlags(renderFlags());
 
     presentationView->jumpToPage(currentPage(), false);
 
@@ -1416,15 +1510,6 @@ void DocumentView::on_thumbnails_cropRectChanged()
 
 void DocumentView::on_pages_linkClicked(bool newTab, int page, qreal left, qreal top)
 {
-    page = qMax(page, 1);
-    page = qMin(page, m_pages.count());
-
-    left = left >= 0.0 ? left : 0.0;
-    left = left <= 1.0 ? left : 1.0;
-
-    top = top >= 0.0 ? top : 0.0;
-    top = top <= 1.0 ? top : 1.0;
-
     if(newTab)
     {
         emit linkClicked(page);
@@ -1521,11 +1606,10 @@ void DocumentView::on_pages_zoomToSelectionRequested(int page, const QRectF& rec
     const qreal visibleWidth = m_layout->visibleWidth(viewport()->width());
     const qreal visibleHeight = m_layout->visibleHeight(viewport()->height());
 
-    const qreal displayedWidth = m_pageItems.at(page - 1)->displayedWidth();
-    const qreal displayedHeight = m_pageItems.at(page - 1)->displayedHeight();
+    const QSizeF displayedSize = m_pageItems.at(page - 1)->displayedSize();
 
-    setScaleFactor(qMin(qMin(visibleWidth / displayedWidth / rect.width(),
-                             visibleHeight / displayedHeight / rect.height()),
+    setScaleFactor(qMin(qMin(visibleWidth / displayedSize.width() / rect.width(),
+                             visibleHeight / displayedSize.height() / rect.height()),
                         Defaults::DocumentView::maximumScaleFactor()));
 
     setScaleMode(ScaleFactorMode);
@@ -2030,7 +2114,7 @@ bool DocumentView::printUsingQt(QPrinter* printer, const PrintOptions& printOpti
 void DocumentView::saveLeftAndTop(qreal& left, qreal& top) const
 {
     const PageItem* page = m_pageItems.at(m_currentPage - 1);
-    const QRectF boundingRect = page->boundingRect().translated(page->pos());
+    const QRectF boundingRect = page->uncroppedBoundingRect().translated(page->pos());
 
     const QPointF topLeft = mapToScene(viewport()->rect().topLeft());
 
@@ -2087,7 +2171,9 @@ void DocumentView::loadFallbackOutline()
         QStandardItem* item = new QStandardItem(tr("Page %1").arg(pageLabelFromNumber(page)));
         item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
 
-        item->setData(page, Qt::UserRole + 1);
+        item->setData(page, Model::Document::PageRole);
+        item->setData(qQNaN(), Model::Document::LeftRole);
+        item->setData(qQNaN(), Model::Document::TopRole);
 
         QStandardItem* pageItem = item->clone();
         pageItem->setText(QString::number(page));
@@ -2145,16 +2231,16 @@ void DocumentView::adjustScrollBarPolicy()
     }
 }
 
-void DocumentView::connectVerticalScrollBar()
-{
-    connect(verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(on_verticalScrollBar_valueChanged()));
-}
-
 void DocumentView::disconnectVerticalScrollBar()
 {
     disconnect(verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(on_verticalScrollBar_valueChanged()));
 }
 
+void DocumentView::reconnectVerticalScrollBar()
+{
+    connect(verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(on_verticalScrollBar_valueChanged()));
+}
+
 void DocumentView::prepareDocument(Model::Document* document, const QVector< Model::Page* >& pages)
 {
     m_prefetchTimer->blockSignals(true);
@@ -2166,12 +2252,12 @@ void DocumentView::prepareDocument(Model::Document* document, const QVector< Mod
     qDeleteAll(m_pageItems);
     qDeleteAll(m_thumbnailItems);
 
-    delete m_document;
-    m_document = document;
-
     qDeleteAll(m_pages);
     m_pages = pages;
 
+    delete m_document;
+    m_document = document;
+
     if(!m_autoRefreshWatcher->files().isEmpty())
     {
         m_autoRefreshWatcher->removePaths(m_autoRefreshWatcher->files());
@@ -2212,7 +2298,6 @@ void DocumentView::preparePages()
     {
         PageItem* page = new PageItem(m_pages.at(index), index);
 
-        page->setInvertColors(m_invertColors);
         page->setRubberBandMode(m_rubberBandMode);
 
         scene()->addItem(page);
@@ -2242,8 +2327,6 @@ void DocumentView::prepareThumbnails()
     {
         ThumbnailItem* page = new ThumbnailItem(m_pages.at(index), pageLabelFromNumber(index + 1), index);
 
-        page->setInvertColors(m_invertColors);
-
         m_thumbnailsScene->addItem(page);
         m_thumbnailItems.append(page);
 
@@ -2265,7 +2348,7 @@ void DocumentView::prepareBackground()
     {
         backgroundColor = s_settings->pageItem().paperColor();
 
-        if(m_invertColors)
+        if(invertColors())
         {
             backgroundColor.setRgb(~backgroundColor.rgb());
         }
@@ -2277,39 +2360,37 @@ void DocumentView::prepareBackground()
 
 void DocumentView::prepareScene()
 {
-    // prepare scale factor and rotation
+    // prepare render parameters and adjust scale factor
+
+    RenderParam renderParam(logicalDpiX(), logicalDpiY(), 1.0,
+                            scaleFactor(), rotation(), renderFlags());
+
+#if QT_VERSION >= QT_VERSION_CHECK(5,1,0)
+
+    if(s_settings->pageItem().useDevicePixelRatio())
+    {
+        renderParam.setDevicePixelRatio(devicePixelRatio());
+    }
+
+#endif // QT_VERSION
 
     const qreal visibleWidth = m_layout->visibleWidth(viewport()->width());
     const qreal visibleHeight = m_layout->visibleHeight(viewport()->height());
 
     foreach(PageItem* page, m_pageItems)
     {
-#if QT_VERSION >= QT_VERSION_CHECK(5,1,0)
+        const QSizeF displayedSize = page->displayedSize(renderParam);
 
-        page->setDevicePixelRatio(devicePixelRatio());
-
-#endif // QT_VERSION
-
-        page->setResolution(logicalDpiX(), logicalDpiY());
-
-        page->setRotation(m_rotation);
-
-        const qreal displayedWidth = page->displayedWidth();
-        const qreal displayedHeight = page->displayedHeight();
-
-        switch(m_scaleMode)
+        if(m_scaleMode == FitToPageWidthMode)
         {
-        default:
-        case ScaleFactorMode:
-            page->setScaleFactor(m_scaleFactor);
-            break;
-        case FitToPageWidthMode:
-            page->setScaleFactor(visibleWidth / displayedWidth);
-            break;
-        case FitToPageSizeMode:
-            page->setScaleFactor(qMin(visibleWidth / displayedWidth, visibleHeight / displayedHeight));
-            break;
+            adjustScaleFactor(renderParam, visibleWidth / displayedSize.width());
         }
+        else if(m_scaleMode == FitToPageSizeMode)
+        {
+            adjustScaleFactor(renderParam, qMin(visibleWidth / displayedSize.width(), visibleHeight / displayedSize.height()));
+        }
+
+        page->setRenderParam(renderParam);
     }
 
     // prepare layout
@@ -2324,17 +2405,17 @@ void DocumentView::prepareScene()
     scene()->setSceneRect(left, 0.0, right - left, height);
 }
 
-void DocumentView::prepareView(qreal changeLeft, qreal changeTop, int visiblePage)
+void DocumentView::prepareView(qreal newLeft, qreal newTop, bool forceScroll, int scrollToPage)
 {
-    qreal left = scene()->sceneRect().left();
-    qreal top = scene()->sceneRect().top();
-    qreal width = scene()->sceneRect().width();
-    qreal height = scene()->sceneRect().height();
+    const QRectF sceneRect = scene()->sceneRect();
+
+    qreal top = sceneRect.top();
+    qreal height = sceneRect.height();
 
     int horizontalValue = 0;
     int verticalValue = 0;
 
-    visiblePage = visiblePage == 0 ? m_currentPage : visiblePage;
+    scrollToPage = scrollToPage != 0 ? scrollToPage : m_currentPage;
 
     const int highlightIsOnPage = m_currentResult.isValid() ? pageOfResult(m_currentResult) : 0;
     const bool highlightCurrentThumbnail = s_settings->documentView().highlightCurrentThumbnail();
@@ -2342,7 +2423,6 @@ void DocumentView::prepareView(qreal changeLeft, qreal changeTop, int visiblePag
     for(int index = 0; index < m_pageItems.count(); ++index)
     {
         PageItem* page = m_pageItems.at(index);
-        const QRectF boundingRect = page->boundingRect().translated(page->pos());
 
         if(m_continuousMode)
         {
@@ -2354,6 +2434,8 @@ void DocumentView::prepareView(qreal changeLeft, qreal changeTop, int visiblePag
             {
                 page->setVisible(true);
 
+                const QRectF boundingRect = page->boundingRect().translated(page->pos());
+
                 top = boundingRect.top() - s_settings->documentView().pageSpacing();
                 height = boundingRect.height() + 2.0 * s_settings->documentView().pageSpacing();
             }
@@ -2365,10 +2447,12 @@ void DocumentView::prepareView(qreal changeLeft, qreal changeTop, int visiblePag
             }
         }
 
-        if(index == visiblePage - 1)
+        if(index == scrollToPage - 1)
         {
-            horizontalValue = qFloor(boundingRect.left() + changeLeft * boundingRect.width());
-            verticalValue = qFloor(boundingRect.top() + changeTop * boundingRect.height());
+            const QRectF boundingRect = page->uncroppedBoundingRect().translated(page->pos());
+
+            horizontalValue = qFloor(boundingRect.left() + newLeft * boundingRect.width());
+            verticalValue = qFloor(boundingRect.top() + newTop * boundingRect.height());
         }
 
         if(index == highlightIsOnPage - 1)
@@ -2382,16 +2466,51 @@ void DocumentView::prepareView(qreal changeLeft, qreal changeTop, int visiblePag
         m_thumbnailItems.at(index)->setHighlighted(highlightCurrentThumbnail && (index == m_currentPage - 1));
     }
 
-    setSceneRect(left, top, width, height);
+    setSceneRect(sceneRect.left(), top, sceneRect.width(), height);
 
-    horizontalScrollBar()->setValue(horizontalValue);
-    verticalScrollBar()->setValue(verticalValue);
+    if(!forceScroll && s_settings->documentView().minimalScrolling())
+    {
+        setValueIfNotVisible(horizontalScrollBar(), horizontalValue);
+        setValueIfNotVisible(verticalScrollBar(), verticalValue);
+    }
+    else
+    {
+        horizontalScrollBar()->setValue(horizontalValue);
+        verticalScrollBar()->setValue(verticalValue);
+    }
 
     viewport()->update();
 }
 
 void DocumentView::prepareThumbnailsScene()
 {
+    // prepare render parameters and adjust scale factor
+
+    RenderParam renderParam(logicalDpiX(), logicalDpiY(), 1.0,
+                            scaleFactor(), rotation(), renderFlags());
+
+#if QT_VERSION >= QT_VERSION_CHECK(5,1,0)
+
+    if(s_settings->pageItem().useDevicePixelRatio())
+    {
+        renderParam.setDevicePixelRatio(devicePixelRatio());
+    }
+
+#endif // QT_VERSION
+
+    const qreal thumbnailSize = s_settings->documentView().thumbnailSize();
+
+    foreach(ThumbnailItem* page, m_thumbnailItems)
+    {
+        const QSizeF displayedSize = page->displayedSize(renderParam);
+
+        adjustScaleFactor(renderParam, qMin(thumbnailSize / displayedSize.width(), thumbnailSize / displayedSize.height()));
+
+        page->setRenderParam(renderParam);
+    }
+
+    // prepare layout
+
     const qreal thumbnailSpacing = s_settings->documentView().thumbnailSpacing();
 
     qreal left = 0.0;
@@ -2405,6 +2524,8 @@ void DocumentView::prepareThumbnailsScene()
     {
         ThumbnailItem* page = m_thumbnailItems.at(index);
 
+        // prepare visibility
+
         if(limitThumbnailsToResults && s_searchModel->hasResults(this) && !s_searchModel->hasResultsOnPage(this, index + 1))
         {
             page->setVisible(false);
@@ -2416,20 +2537,7 @@ void DocumentView::prepareThumbnailsScene()
 
         page->setVisible(true);
 
-        // prepare scale factor
-
-#if QT_VERSION >= QT_VERSION_CHECK(5,1,0)
-
-        page->setDevicePixelRatio(devicePixelRatio());
-
-#endif // QT_VERSION
-
-        page->setResolution(logicalDpiX(), logicalDpiY());
-
-        page->setScaleFactor(qMin(s_settings->documentView().thumbnailSize() / page->displayedWidth(),
-                                  s_settings->documentView().thumbnailSize() / page->displayedHeight()));
-
-        // prepare layout
+        // prepare position
 
         const QRectF boundingRect = page->boundingRect();
 
@@ -2470,7 +2578,7 @@ void DocumentView::prepareHighlight(int index, const QRectF& rect)
 
     disconnectVerticalScrollBar();
     centerOn(m_highlight);
-    connectVerticalScrollBar();
+    reconnectVerticalScrollBar();
 
     viewport()->update();
 }
