@@ -34,7 +34,7 @@ class QFileSystemWatcher;
 class QPrinter;
 class QStandardItemModel;
 
-#include "global.h"
+#include "renderparam.h"
 #include "printoptions.h"
 
 namespace qpdfview
@@ -104,11 +104,21 @@ public:
     Rotation rotation() const { return m_rotation; }
     void setRotation(Rotation rotation);
 
-    bool invertColors() const { return m_invertColors; }
-    void setInvertColors(bool invertColors);
+    qpdfview::RenderFlags renderFlags() const { return m_renderFlags; }
+    void setRenderFlags(qpdfview::RenderFlags renderFlags);
+    void setRenderFlag(qpdfview::RenderFlag renderFlag, bool enabled = true);
 
-    bool convertToGrayscale() const { return m_convertToGrayscale; }
-    void setConvertToGrayscale(bool convertToGrayscale);
+    bool invertColors() const { return m_renderFlags.testFlag(InvertColors); }
+    void setInvertColors(bool invertColors) { setRenderFlag(InvertColors, invertColors); }
+
+    bool convertToGrayscale() const { return m_renderFlags.testFlag(ConvertToGrayscale); }
+    void setConvertToGrayscale(bool convertToGrayscale) { setRenderFlag(ConvertToGrayscale, convertToGrayscale); }
+
+    bool trimMargins() const { return m_renderFlags.testFlag(TrimMargins); }
+    void setTrimMargins(bool trimMargins) { setRenderFlag(TrimMargins, trimMargins); }
+
+    CompositionMode compositionMode() const;
+    void setCompositionMode(CompositionMode compositionMode);
 
     bool highlightAll() const { return m_highlightAll; }
     void setHighlightAll(bool highlightAll);
@@ -155,8 +165,14 @@ signals:
     void linkClicked(int page);
     void linkClicked(bool newTab, const QString& filePath, int page);
 
+    void renderFlagsChanged(qpdfview::RenderFlags renderFlags);
+
     void invertColorsChanged(bool invertColors);
     void convertToGrayscaleChanged(bool convertToGrayscale);
+    void trimMarginsChanged(bool trimMargins);
+
+    void compositionModeChanged(CompositionMode compositionMode);
+
     void highlightAllChanged(bool highlightAll);
     void rubberBandModeChanged(RubberBandMode rubberBandMode);
 
@@ -176,7 +192,7 @@ public slots:
     void firstPage();
     void lastPage();
 
-    void jumpToPage(int page, bool trackChange = true, qreal changeLeft = 0.0, qreal changeTop = 0.0);
+    void jumpToPage(int page, bool trackChange = true, qreal newLeft = qQNaN(), qreal newTop = qQNaN());
 
     bool canJumpBackward() const;
     void jumpBackward();
@@ -289,8 +305,8 @@ private:
     qreal m_scaleFactor;
     Rotation m_rotation;
 
-    bool m_invertColors;
-    bool m_convertToGrayscale;
+    qpdfview::RenderFlags m_renderFlags;
+
     bool m_highlightAll;
     RubberBandMode m_rubberBandMode;
 
@@ -311,8 +327,8 @@ private:
     void loadDocumentDefaults();
 
     void adjustScrollBarPolicy();
-    void connectVerticalScrollBar();
     void disconnectVerticalScrollBar();
+    void reconnectVerticalScrollBar();
 
     void prepareDocument(Model::Document* document, const QVector< Model::Page* >& pages);
     void preparePages();
@@ -320,7 +336,7 @@ private:
     void prepareBackground();
 
     void prepareScene();
-    void prepareView(qreal changeLeft = 0.0, qreal changeTop = 0.0, int visiblePage = 0);
+    void prepareView(qreal newLeft = 0.0, qreal newTop = 0.0, bool forceScroll = true, int scrollToPage = 0);
 
     void prepareThumbnailsScene();
 
