@@ -1,6 +1,7 @@
 /*
 
-Copyright 2012-2013 Adam Reichold
+Copyright 2015 S. Razi Alavizadeh
+Copyright 2012-2015 Adam Reichold
 Copyright 2012 Alexander Volkov
 
 This file is part of qpdfview.
@@ -61,7 +62,10 @@ public:
         int tileSize() const { return m_tileSize; }
 
         const QIcon& progressIcon() const { return m_progressIcon; }
+        void setProgressIcon(const QIcon& progressIcon) { m_progressIcon = progressIcon; }
+
         const QIcon& errorIcon() const { return m_errorIcon; }
+        void setErrorIcon(const QIcon& errorIcon) { m_errorIcon = errorIcon; }
 
         bool keepObsoletePixmaps() const { return m_keepObsoletePixmaps; }
         void setKeepObsoletePixmaps(bool keepObsoletePixmaps);
@@ -69,9 +73,6 @@ public:
         bool useDevicePixelRatio() const { return m_useDevicePixelRatio; }
         void setUseDevicePixelRatio(bool useDevicePixelRatio);
 
-        bool trimMargins() const { return m_trimMargins; }
-        void setTrimMargins(bool trimMargins);
-
         bool decoratePages() const { return m_decoratePages; }
         void setDecoratePages(bool decoratePages);
 
@@ -125,8 +126,6 @@ public:
         bool m_keepObsoletePixmaps;
         bool m_useDevicePixelRatio;
 
-        bool m_trimMargins;
-
         bool m_decoratePages;
         bool m_decorateLinks;
         bool m_decorateFormFields;
@@ -186,6 +185,9 @@ public:
         int pagesPerRow() const { return m_pagesPerRow; }
         void setPagesPerRow(int pagesPerRow);
 
+        bool minimalScrolling() const { return m_minimalScrolling; }
+        void setMinimalScrolling(bool minimalScrolling);
+
         bool highlightCurrentThumbnail() const { return m_highlightCurrentThumbnail; }
         void setHighlightCurrentThumbnail(bool highlightCurrentThumbnail);
 
@@ -254,6 +256,12 @@ public:
         bool convertToGrayscale() const;
         void setConvertToGrayscale(bool convertToGrayscale);
 
+        bool trimMargins() const;
+        void setTrimMargins(bool trimMargins);
+
+        CompositionMode compositionMode() const;
+        void setCompositionMode(CompositionMode compositionMode);
+
         bool highlightAll() const;
         void setHighlightAll(bool highlightAll);
 
@@ -268,6 +276,8 @@ public:
 
         int m_pagesPerRow;
 
+        bool m_minimalScrolling;
+
         bool m_highlightCurrentThumbnail;
         bool m_limitThumbnailsToResults;
 
@@ -352,6 +362,12 @@ public:
         QStringList viewToolBar() const;
         void setViewToolBar(const QStringList& viewToolBar);
 
+        QStringList documentContextMenu() const;
+        void setDocumentContextMenu(const QStringList& documentContextMenu);
+
+        QStringList tabContextMenu() const;
+        void setTabContextMenu(const QStringList& tabContextMenu);
+
         bool scrollableMenus() const;
         void setScrollableMenus(bool scrollableMenus);
 
@@ -480,8 +496,6 @@ public:
         static bool keepObsoletePixmaps() { return false; }
         static bool useDevicePixelRatio() { return false; }
 
-        static bool trimMargins() { return false; }
-
         static bool decoratePages() { return true; }
         static bool decorateLinks() { return true; }
         static bool decorateFormFields() { return true; }
@@ -533,6 +547,8 @@ public:
 
         static int pagesPerRow() { return 3; }
 
+        static bool minimalScrolling() { return false; }
+
         static bool highlightCurrentThumbnail() { return false; }
         static bool limitThumbnailsToResults() { return false; }
 
@@ -546,6 +562,8 @@ public:
 
         static qreal thumbnailSize() { return 150.0; }
 
+        static CompositionMode compositionMode() { return DefaultCompositionMode; }
+
         static bool matchCase() { return false; }
         static bool wholeWords() { return false; }
 
@@ -567,7 +585,9 @@ public:
         static Rotation rotation() { return RotateBy0; }
 
         static bool invertColors() { return false; }
-        static bool convertToGrayscale() { return false; }
+        static bool convertToGrayscale() { return false; }        
+        static bool trimMargins() { return false; }
+
         static bool highlightAll() { return false; }
 
     private:
@@ -613,6 +633,9 @@ public:
         static QStringList editToolBar() { return QStringList() << "currentPage" << "previousPage" << "nextPage"; }
         static QStringList viewToolBar() { return QStringList() << "scaleFactor" << "zoomIn" << "zoomOut"; }
 
+        static QStringList documentContextMenu() { return QStringList() << "previousPage" << "nextPage" << "firstPage" << "lastPage" << "separator" << "jumpToPage" << "jumpBackward" << "jumpForward" << "separator" << "setFirstPage" << "separator" << "findPrevious" << "findNext" << "cancelSearch"; }
+        static QStringList tabContexntMenu() { return QStringList() << "openCopyInNewTab" << "openContainingFolder" << "separator" << "closeAllTabs" << "closeAllTabsButThisOne" << "closeAllTabsToTheLeft" << "closeAllTabsToTheRight"; }
+
         static bool scrollableMenus() { return false; }
         static bool searchableMenus() { return false; }
 
