@@ -34,7 +34,8 @@ Settings* TileItem::s_settings = 0;
 
 QCache< TileItem::CacheKey, TileItem::CacheObject > TileItem::s_cache;
 
-TileItem::TileItem(QObject* parent) : QObject(parent),
+TileItem::TileItem(PageItem* page) : QObject(page),
+    m_page(page),
     m_rect(),
     m_cropRect(),
     m_pixmapError(false),
@@ -50,7 +51,7 @@ TileItem::TileItem(QObject* parent) : QObject(parent),
 
     s_cache.setMaxCost(s_settings->pageItem().cacheSize());
 
-    m_renderTask = new RenderTask(parentPage()->m_page, this);
+    m_renderTask = new RenderTask(m_page->m_page, this);
 
     connect(m_renderTask, SIGNAL(finished()), SLOT(on_renderTask_finished()));
     connect(m_renderTask, SIGNAL(imageReady(RenderParam,QRect,bool,QImage,QRectF)), SLOT(on_renderTask_imageReady(RenderParam,QRect,bool,QImage,QRectF)));
@@ -64,7 +65,7 @@ TileItem::~TileItem()
 
 void TileItem::setCropRect(const QRectF& cropRect)
 {
-    if(!s_settings->pageItem().trimMargins())
+    if(!m_page->m_renderParam.trimMargins())
     {
         return;
     }
@@ -73,7 +74,7 @@ void TileItem::setCropRect(const QRectF& cropRect)
     {
         m_cropRect = cropRect;
 
-        parentPage()->updateCropRect();
+        m_page->updateCropRect();
     }
 }
 
@@ -166,9 +167,7 @@ int TileItem::startRender(bool prefetch)
         return 0;
     }
 
-    m_renderTask->start(parentPage()->m_renderParam,
-                        m_rect, prefetch,
-                        s_settings->pageItem().trimMargins(), s_settings->pageItem().paperColor());
+    m_renderTask->start(m_page->m_renderParam, m_rect, prefetch);
 
     return 1;
 }
@@ -200,14 +199,10 @@ void TileItem::on_renderTask_finished()
     if(m_deleteAfterRender)
     {
         deleteLater();
-        return;
     }
-
-    PageItem* page = parentPage();
-
-    if(!s_settings->pageItem().useTiling() || page->thumbnailMode() || page->m_exposedTileItems.contains(this))
+    else if(!m_page->useTiling() || m_page->m_exposedTileItems.contains(this))
     {
-        page->update();
+        m_page->update();
     }
 }
 
@@ -215,7 +210,7 @@ void TileItem::on_renderTask_imageReady(const RenderParam& renderParam,
                                         const QRect& rect, bool prefetch,
                                         QImage image, QRectF cropRect)
 {
-    if(parentPage()->m_renderParam != renderParam || m_rect != rect)
+    if(m_page->m_renderParam != renderParam || m_rect != rect)
     {
         return;
     }
@@ -231,7 +226,7 @@ void TileItem::on_renderTask_imageReady(const RenderParam& renderParam,
 
     if(prefetch && !m_renderTask->wasCanceledForcibly())
     {
-        const int cost = image.width() * image.height() * image.depth() / 8;
+        const int cost = qMax(1, image.width() * image.height() * image.depth() / 8);
         s_cache.insert(cacheKey(), new CacheObject(QPixmap::fromImage(image), cropRect), cost);
 
         setCropRect(cropRect);
@@ -244,26 +239,14 @@ void TileItem::on_renderTask_imageReady(const RenderParam& renderParam,
     }
 }
 
-inline PageItem* TileItem::parentPage() const
-{
-    return qobject_cast< PageItem* >(parent());
-}
-
 inline TileItem::CacheKey TileItem::cacheKey() const
 {
-    PageItem* page = parentPage();
     QByteArray key;
+    QDataStream stream(&key, QIODevice::WriteOnly);
 
-    QDataStream(&key, QIODevice::WriteOnly)
-            << page->m_renderParam.resolution.resolutionX
-            << page->m_renderParam.resolution.resolutionY
-            << page->m_renderParam.scaleFactor
-            << page->m_renderParam.rotation
-            << page->m_renderParam.invertColors
-            << page->m_renderParam.convertToGrayscale
-            << m_rect;
+    stream << m_page->m_renderParam << m_rect;
 
-    return qMakePair(page, key);
+    return qMakePair(m_page, key);
 }
 
 QPixmap TileItem::takePixmap()
@@ -282,7 +265,7 @@ QPixmap TileItem::takePixmap()
 
     if(!m_pixmap.isNull())
     {
-        int cost = m_pixmap.width() * m_pixmap.height() * m_pixmap.depth() / 8;
+        const int cost = qMax(1, m_pixmap.width() * m_pixmap.height() * m_pixmap.depth() / 8);
         s_cache.insert(key, new CacheObject(m_pixmap, m_cropRect), cost);
 
         pixmap = m_pixmap;
